#include <glib.h>
#include <glib/gprintf.h>

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include <sys/types.h> /* stat */
#include <sys/stat.h>

#include <string.h>

//#include <errno.h>

#define MAXLINE 80

#define PROGNAME "lircd-conf-generator"
#define TEXTBEGIN "\
#\n\
# this config file was automatically generated\n\
# using lircd-conf-generator\n\
#\n\
# brand:             SIMPLE UDP\n\
# model:             Could be many\n\
# supported devices: sudp\n\
#\n\
# comment:\n\
#\n\
\n\
begin remote\n\
\n\
\tname  SUDP-CONTROL\n\
\tbits  16\n\
\n\
\t\tgap  100000\n\
\t\trepeat_bit 0\n\
\n\
\t\tbegin codes\n"

#define TEXTEND "\
\t\tend codes\n\
end remote\n"

struct r2loptions
{
  gint verbose;
  gint force;
  gchar *keyfile;
  gchar *lircfile;
};


static gint check_files (struct r2loptions *options);
static void print_help (void);
static int parse_options (int argc, char **argv, struct r2loptions *options);
static gchar **read_lcontrol_keys (guint *numkeys, struct r2loptions *options);
static gint write_keys_to_lircfile (const gchar **keys, gint numkeys, 
				    struct r2loptions *options);


static void
print_help (void) 
{
  g_print ("\nUsage: %s -c <configfile> -l <lircdfile>\n", PROGNAME);
  g_print ("Example: %s -c /home/user/.lcontrol/config -l lircd.conf\n\n", 
	   PROGNAME);
  g_print ("Converts buttons from config file to lircd.conf file.\n\n");
  g_print ("Options:\n");
  g_print ("  -c, --configfile\t lcontrol configfile\n");
  g_print ("  -l, --lircfile\t lircd.conf outputfile\n");
  g_print ("  -f, --force\t\t force to overwrite lircd configure\n");
  g_print ("  -v, --verbose\t\t verbose output\n");
  g_print ("  -h, --help\t\t prints this help\n");
  g_print ("\n");
  g_print ("Report bugs to <jvaltane <at> kapsi.fi>\n");
  g_print ("\n");

  exit (0);
}

static int 
parse_options (int argc, char **argv, struct r2loptions *options )
{
  int cmd = 0;

  while (1) {
    int option_index = 0;

    static struct option long_options[] =
    {
      {"verbose",      no_argument, 0, 'v'},
      {"help",      no_argument, 0, 'h'},
      {"force",      no_argument, 0, 'f'},
      {"configfile",   required_argument, 0, 'c'},
      {"lircfile",  required_argument, 0, 'l'},
      {0, 0, 0, 0}
    };

    cmd = getopt_long (argc, argv, "vfhc:l:",
                       long_options, &option_index);

    if (cmd == -1)
      break;

    switch (cmd) {
      case 0:
        if (long_options[option_index].flag != 0)
          break;
        printf ("option %s", long_options[option_index].name);
        if (optarg)
           printf (" with arg %s", optarg);
        printf ("\n");
        break;
      case 'h':
	print_help ();
        break;
      case 'f':
	options->force = 1;
        break;
      case 'v':
	options->verbose = 1;
        break;
      case 'l':
        printf ("option -l with value `%s'\n", optarg);
	options->lircfile = g_strdup (optarg);
        break;
      case 'c':
        printf ("option -c with value `%s'\n", optarg);
	options->keyfile = g_strdup (optarg);
        break;
      case '?':
        break;
      default:
        printf ("abort");
    }
  }
  return 0;
}

static gchar **
read_lcontrol_keys (guint *numkeys, struct r2loptions *options)
{
  GError *error = NULL;
  gchar **keys = NULL;

  GKeyFile *keyfile = NULL;
  GKeyFileFlags flags = G_KEY_FILE_KEEP_TRANSLATIONS;


  keyfile = g_key_file_new ();
  if (keyfile == NULL)
    return NULL;

  if (options->verbose != 0)
    g_print ("Opening configfile : %s..\n", options->keyfile);
  g_key_file_load_from_file (keyfile, options->keyfile, flags, &error);

  if (error != NULL) {
    g_error ("Error opening keyfile: %s", error->message);
    g_clear_error (&error);
    error = NULL;
    return NULL;
  }

  if (options->verbose != 0)
    g_print ("getting data...\n");

  /* Only keys are needed */
  keys = g_key_file_get_keys (keyfile, "buttons", numkeys, &error);
  if (error != NULL) {
    g_error ("Error reading keys: %s", error->message);
    g_clear_error (&error);
    g_key_file_free (keyfile);
    error = NULL;
    return NULL;
  }

  if (keyfile != NULL)
    g_key_file_free (keyfile);

  return keys;
}

static gint
check_files (struct r2loptions *options)
{
  struct stat *st = malloc (sizeof (struct stat));
  int retval = 0;

  /* Checks to the keyfile */
  if ((retval = stat (options->keyfile, st)) != 0 ) {
    g_print ("Could not open keyfile\n");
    if (options->verbose) {
      g_print ("Keyfile is not real file\n");
    }
    free (st);
    return 1;
  } else {
    /* Check is file readable */

  }

  /* Checks to the lircfile */
  if (options->force == 0) {
    if (stat (options->lircfile, st) == 0) {
      g_print ("There is lircd config allready\n");
      g_print ("Use `-f' option to overwrite\n");
      free (st);
      return 2;
    }
  } else {
    /* check is file writable*/
  }

  free (st);
  return 0;
}


static gint
write_keys_to_lircfile (const gchar **keys, gint numkeys, 
			struct r2loptions *options)
{
  FILE *fwrite = NULL;
  int retval = 0;
  guint i = 0;
  gchar *tmpstr = NULL;
  gchar tmpstr2[MAXLINE];

  if (options->verbose != 0)
    g_print ("Creating file to write\n");

  fwrite = fopen ((const char *)options->lircfile, "w");
  if (fwrite == NULL) {
    if (options->verbose != 0)
      g_print ("fopen fails!!!\n");
    return 1;
  }

  if (options->verbose != 0)
    g_print ("Writing to file\n");

  retval = fputs(TEXTBEGIN, fwrite);
  if (retval < 0 || retval == EOF) {
    if (options->verbose != 0)
      g_print ("Writing of the start the file fails!!!\n");
    return 2;
  }

  for (i=0; i<numkeys; i++) {
    g_debug ("key[%d]: %s",i, keys[i]);
    tmpstr = g_ascii_strup (keys[i], strlen (keys[i]));
    g_debug ("tmpstr");
    g_debug ("%s", tmpstr);

    g_snprintf (tmpstr2, MAXLINE-1, "\t\t\t%s\t0x0000%04x\n", tmpstr, 
		(short)i+1);
    retval = fputs(tmpstr2, fwrite);
    if (retval < 0 || retval == EOF) {
      if (options->verbose != 0)
	g_print ("Writing of the codes fails!!!\n");
      return 3;
    }
    g_free (tmpstr);
    tmpstr = NULL;
  }

  retval = fputs(TEXTEND, fwrite);
  if (retval < 0 || retval == EOF) {
    if (options->verbose != 0)
      g_print ("Writing of the end of the file fails!!!\n");
    return 4;
  }

  return 0;
}

int
main (int argc, char **argv)
{
  struct r2loptions options = {0, 0, NULL, NULL};

  guint numkeys = 0;
  gchar **keys;

  /* First have to parse options */
  parse_options (argc, argv, &options);
  if ( (options.keyfile == NULL) || (options.lircfile == NULL)) {
    g_print ("Usage: %s -c <keyfile> -l <lircfile>\n", PROGNAME);
    g_print ("Try `%s -h' for more information.\n", PROGNAME);
  }

  if (check_files (&options) != 0) {
    g_free (options.keyfile);
    g_free (options.lircfile);
    return 1;
  }

  /* read keys */
  keys = read_lcontrol_keys (&numkeys, &options );
  if(keys == NULL || numkeys == 0) {
    g_print ("Could not read keys!!!\n");
    g_free (options.keyfile);
    g_free (options.lircfile);
    return 2;
  }

  if (write_keys_to_lircfile ((const char **)keys, numkeys, &options) != 0) {
    g_print ("Could not write to file!!!\n");
    g_strfreev (keys);
    g_free (options.keyfile);
    g_free (options.lircfile);
    return 3;
  }

  g_strfreev (keys);
  g_free (options.keyfile);
  g_free (options.lircfile);

  return 0;

}
